using System;
using System.Collections;

/*
 * TODO
 * + XML parsen in UdpReceiver
 * + XMLPackaging.xml2ave u. ave2xml um Attribute erweitern
 * + UdpReceiver.receiveBroadcast() als Thread wie UdpSender
 * + Identcard mit PIN entschlsseln
 * - Bildinformation (als XML!) bertragen und anzeigen
 * - Tcp-Verbindung impl.
 * - weitere XML-Pakete fr Protokoll definieren
 * - Kontierungsdaten verwalten
 * - Dialog fr Abrechnung erstellen
 * - WLAN-Api: Einstellungen durchfhren und restaurieren, mind. aber berprfen
 * - CAB-Archiv erzeugen
 */
namespace viaticum
{
	/// <summary>
	/// Context info
	/// </summary>
	public class Context
	{
		public enum EnumDirections {Westen, Osten, Norden, Sden};
		public enum EnumGender {mnnlich, weiblich, unspezifiziert};
		public enum EnumSeat {hinten, vorne, unspezifiziert};
		public enum EnumServiceType {Fahrer, Mitfahrer, unspezifiziert};

		public const int minAge = 0;
		public const int maxAge = 200;
		public const int maxCarAge = 100;
		public const int defNumPers = 0;
		public const int defMaxNumPers = 1;

		private static TimeSpan max_lifetime_partner = new TimeSpan( 0, 0, 0, 8, 0 );  // max. 8 seconds respond time
		private static bool havePartner = false;
		private static string reqDestination1;
		private static string reqDestination2;
		private static bool   reqDestination2Optional;
		private static EnumDirections reqDirection;
		private static EnumGender reqGender;
		private static int reqAgeFrom;
		private static int reqAgeTo;
		private static bool reqNonSmoker;
		private static bool reqMaySmoke;
		private static bool reqHasAirbag;
		private static bool reqHasAbs;
		private static int reqMaxNumPers;
		private static int reqNumPers;
		private static EnumSeat reqSeat;
		private static int reqAgeCar;
		private static Smartcard identcard;
		private static ArrayList partnerAddresses = new ArrayList();
		private static bool loggedIn = false;		

		public static void addPartnerAddress(string ipAddress) 
		{
			PartnerIPAddress partner = new PartnerIPAddress(ipAddress, DateTime.Now);
			partnerAddresses.Add(partner);
			havePartner = true;
		}

		public static void getPartnerAddress(out string ipAddress) 
		{
			ipAddress = null;
			DateTime now = DateTime.Now;

			foreach (PartnerIPAddress partner in partnerAddresses) 
			{
				if (partner.Ts_alive.Add(max_lifetime_partner) > now) 
				{
					partnerAddresses.Remove(partner);
				} 
				else 
				{
					ipAddress = partner.IpAddress;					
					break;
				}
			}
			havePartner = (partnerAddresses.Count > 0); // race condition??
		}

		public static bool LoggedIn
		{
			get { return loggedIn; }
			set { loggedIn = value; }
		}

		public static Smartcard Identcard
		{
			get { return identcard; }
			set { identcard = value; }
		}

		public static int ReqAgeCar 
		{
			get { return reqAgeCar; }
			set { reqAgeCar = value; }
		}

		public static bool ReqHasAbs 
		{
			get { return reqHasAbs; }
			set { reqHasAbs = value; }
		}

		public static bool HavePartner
		{
			get { return havePartner; }
		}

		public static int ReqNumPers 
		{
			get { return reqNumPers;}
			set {reqNumPers = value;}
		}
		
		public static int ReqMaxNumPers 
		{
			get { return reqMaxNumPers;}
			set {reqMaxNumPers = value;}
		}

		public static bool ReqHasAirbag 
		{
			get { return reqHasAirbag; }
			set { reqHasAirbag = value; }
		}
		public static bool ReqNonSmoker 
		{
			get { return reqNonSmoker; }
			set { reqNonSmoker = value; }
		}

		public static bool ReqMaySmoke 
		{
			get { return reqMaySmoke; }
			set { reqMaySmoke = value; }
		}

		public static string ReqDestination1 
		{
			get { return reqDestination1; }
			set { reqDestination1 = value; }
		}

		public static string ReqDestination2 
		{
			get { return reqDestination2; }
			set { reqDestination2 = value; }
		}

		public static bool ReqDestination2Optional 
		{
			get { return reqDestination2Optional; }
			set { reqDestination2Optional = value; }
		}

		public static EnumDirections ReqDirection 
		{
			get { return reqDirection;}
			set { reqDirection = value;}
		}

		public static Context.EnumServiceType string2ServiceType(string s) 
		{
			if (s.Equals("Fahrer")) 
			{
				return (Context.EnumServiceType.Fahrer);
			} 
			else 
			{
				return (Context.EnumServiceType.Mitfahrer);
			}
		}
		
		public static string bool2string(bool b) 
		{
			return ((b) ? "Y" : "N");
		}
		
		public static bool string2bool(string s) 
		{
			return (s.Equals("Y"));
		}

		public static string seat2string(Context.EnumSeat s) 
		{
			switch (s) 
			{
				case Context.EnumSeat.vorne:
					return("v");
				case Context.EnumSeat.hinten:
					return("h");
				default:
					return("u");
			}
		}

		public static Context.EnumSeat string2seat(string s) {
			if (s.Equals("v")) 
			{
				return (Context.EnumSeat.vorne);
			} 
			else if (s.Equals("h")) 
			{
				return (Context.EnumSeat.hinten);
			} 
			else 
			{
				return (Context.EnumSeat.unspezifiziert);
			}
		}

		public static string gender2string(Context.EnumGender g) 
		{
			switch (g) 
			{
				case Context.EnumGender.weiblich:
					return ("w");
				case Context.EnumGender.mnnlich:
					return ("m");
				default:
					return ("u");
			}
		}

		public static Context.EnumGender string2gender(string s) 
		{
			if (s.Equals("w")) 
			{
				return(Context.EnumGender.weiblich);
			} 
			else if (s.Equals("m"))
			{
				return(Context.EnumGender.mnnlich);
			} 
			else 
			{
				return(Context.EnumGender.unspezifiziert);
			}
		}

		public static string direction2string(Context.EnumDirections d) 
		{
			switch (d) 
			{
				case Context.EnumDirections.Norden:
					return ("n");
				case Context.EnumDirections.Osten:
					return ("o");
				case Context.EnumDirections.Sden:
					return ("s");
				case Context.EnumDirections.Westen:
					return ("w");
			}
			return null;
		}

		public static Context.EnumDirections string2direction(string s) 
		{
			if (s.Equals("n")) 
			{
				return(Context.EnumDirections.Norden);
			} 
			else if (s.Equals("o")) 
			{
				return(Context.EnumDirections.Osten);
			}
			else if (s.Equals("s")) 
			{
				return(Context.EnumDirections.Sden);
			}
			else if (s.Equals("w")) 
			{
				return(Context.EnumDirections.Westen);
			}

			return Context.EnumDirections.Westen;
		}

		public static string DirectionAsString 
		{
			get 
			{
				return reqDirection.ToString();
			}
			set 
			{
				if (value.Equals("Norden")) 
				{
					reqDirection = EnumDirections.Norden;
				} 
				else if (value.Equals("Osten")) 
				{
					reqDirection = EnumDirections.Osten;
				} 
				else if (value.Equals("Sden")) 
				{
					reqDirection = EnumDirections.Sden;
				} 
				else if (value.Equals("Westen")) 
				{
					reqDirection = EnumDirections.Westen;
				} 

			}
		}
		public static EnumGender ReqGender 
		{
			get 
			{
				return reqGender;
			}
			set 
			{
				reqGender = value;
			}
		}
		public static EnumSeat ReqSeat 
		{
			get 
			{
				return reqSeat;
			}
			set 
			{
				reqSeat = value;
			}
		}

		public static int ReqAgeFrom 
		{
			get { return reqAgeFrom; }
			set { reqAgeFrom = value; }
		}
		public static int ReqAgeTo 
		{
			get { return reqAgeTo; }
			set { reqAgeTo = value; }
		}

		public static void clear() 
		{
			reqAgeCar = maxCarAge;
			reqAgeFrom = minAge;
			reqAgeTo = maxAge;
			reqDestination1 = "";
			reqDestination2 = "";
			reqDestination2Optional = false;
			reqDirection = EnumDirections.Westen;
			reqGender = EnumGender.unspezifiziert;
			reqHasAbs = false;
			reqHasAirbag = false;
			reqNonSmoker = false;
			reqMaySmoke = false;
			reqSeat = EnumSeat.unspezifiziert;
			identcard = null;
			loggedIn = false;
			reqNumPers = defNumPers;    // additional
			reqMaxNumPers = defMaxNumPers; // total
		}

		public static int calcAge(DateTime birthDate) 
		{
			DateTime now = DateTime.Now; 
			int age = now.Year - birthDate.Year;
			if (now.Month < birthDate.Month || (now.Month == birthDate.Month && now.Day < birthDate.Day)) 
			{
				--age;
			}

			return(age);
		}
	}
}
