using System;
using System.IO;
using System.Collections;

namespace viaticum
{
	/// <summary>
	/// Summary description for IniFile.
	/// </summary>
	public class IniFile
	{
		public enum EnumFileMode {read, write};
		private string filename;
		private EnumFileMode fileMode;
		private Hashtable settings;

		void readFile() 
		{
			string line;
			string delimStr = "=";
			char [] delimiter = delimStr.ToCharArray();
			string []splitted;
			StreamReader sr = new StreamReader(filename);

			try 
			{
				while ((line = sr.ReadLine()) != null) 
				{
					splitted = line.Split(delimiter);
					if (splitted[0] != null && !splitted[0].Equals("")) 
					{
						settings.Add(splitted[0], splitted[1]);
					}
				}
			} 
			finally 
			{
				sr.Close();
			}
		}
		public void WriteFile() 
		{
			StreamWriter sw = new StreamWriter(filename);
			try 
			{
				IDictionaryEnumerator myEnumerator = settings.GetEnumerator();
				while ( myEnumerator.MoveNext() ) 
				{
					sw.WriteLine(myEnumerator.Key + "=" + myEnumerator.Value);
				}
				sw.Flush();
			} 
			finally 
			{
				sw.Close();
			}
		}
		public IniFile(string file, EnumFileMode mode)
		{
			filename = file;
			fileMode = mode;
			settings = new Hashtable();
			if (fileMode == EnumFileMode.read) 
			{
				readFile();
			}
		}
		public void Put(string key, string val) 
		{
			if (fileMode == EnumFileMode.read) 
			{
				throw new Exception("IniFile: Put kann nicht im Lesemodus aufgerufen werden");
			}
			settings.Add(key, val);
		}
		public void Put(string key, int val) 
		{
			if (fileMode == EnumFileMode.read) 
			{
				throw new Exception("IniFile: Put kann nicht im Lesemodus aufgerufen werden");
			}
			settings.Add(key, val.ToString());
		}
		public void Put(string key, Int64 val) 
		{
			if (fileMode == EnumFileMode.read) 
			{
				throw new Exception("IniFile: Put kann nicht im Lesemodus aufgerufen werden");
			}
			settings.Add(key, val.ToString());
		}
		public void Put(string key, bool val) 
		{
			if (fileMode == EnumFileMode.read) 
			{
				throw new Exception("IniFile: Put kann nicht im Lesemodus aufgerufen werden");
			}
			settings.Add(key, val ? "TRUE" : "FALSE");
		}
		public void Get(string key, out string val) 
		{
			if (fileMode == EnumFileMode.write) 
			{
				throw new Exception("IniFile: Get kann nicht im Schreibmodus aufgerufen werden");
			}
			if (settings.ContainsKey(key)) 
			{
				val = settings[key].ToString();
			} 
			else 
			{
				val = "";
			}
		}

		public void Get(string key, out bool val) 
		{
			if (fileMode == EnumFileMode.write) 
			{
				throw new Exception("IniFile: Get kann nicht im Schreibmodus aufgerufen werden");
			}
			if (settings.ContainsKey(key)) 
			{
				string s = settings[key].ToString();
				val = s.Equals("TRUE");					
			} 
			else 
			{
				val = false;
			}

		}

		public void Get(string key, out int val) 
		{
				string s;

			if (fileMode == EnumFileMode.write) 
			{
				throw new Exception("IniFile: Get kann nicht im Schreibmodus aufgerufen werden");
			}
			if (settings.ContainsKey(key)) 
			{
				s = settings[key].ToString();
				val = Convert.ToInt32(s);
			} 
			else 
			{
				val = 0;
			}
		}
		public void Get(string key, out Int64 val) 
		{
			string s;

			if (fileMode == EnumFileMode.write) 
			{
				throw new Exception("IniFile: Get kann nicht im Schreibmodus aufgerufen werden");
			}
			if (settings.ContainsKey(key)) 
			{
				s = settings[key].ToString();
				val = Convert.ToInt64(s);
			} 
			else 
			{
				val = 0;
			}
		}


	}
}
