using System;
using System.IO; 
using System.Text; 
using System.Xml; 
using System.Drawing;
using System.Globalization;

namespace viaticum
{
	public class Smartcard
	{
		private UTF8Encoding utf8;
		private string sigPriv;
		private string sigPub;
		private string version;
		private DateTime validFrom;
		private DateTime validTo;
		private string myId;
		private Context.EnumServiceType serviceType;
		private Image myImage;
		private Context.EnumGender myGender;
		private string myCarLicence;
		private string privXml;
		private string pubXml;
		private string mySecKey;
		private string masterPubKey;
		private DateTime birthDate;
		private DateTime carBuilt;
		private bool carHasAirbag;
		private bool carHasAbs;
		private bool nonSmoker;
		private string mySsid;
		private string myIpAddress;
		private string myNetMask;
		private string myMulticastGroup;
		private int myUdpPort;
		private int myTcpPort;
		private int myAge; // calculated from birthDate
		private int myCarAge; // calculated from carBuilt
		
		public string PubXml 
		{
			get {return pubXml;}
		}

		public string SigPub 
		{
			get {return sigPub;}
		}

		public string MySecKey 
		{
			get {return mySecKey;}
		}

		public string MasterPubKey 
		{
			get {return masterPubKey;}
		}


		public string Version 
		{
			get {return version;}
		}

		public DateTime ValidFrom 
		{
			get {return validFrom;}
		}
		
		public DateTime ValidTo 
		{
			get {return validTo;}
		}

			public Image MyImage 
		{
			get { return myImage; }
		}

		public string MyId 
		{
			get { return myId; }
		}

		public int MyTcpPort 
		{
			get { return myTcpPort; }
		}

		public int MyUdpPort 
		{
			get { return myUdpPort; }
		}
		public string MyMulticastGroup 
		{
			get { return myMulticastGroup; }
		}
		public string MyNetMask 
		{
			get { return myNetMask; }
		}
		public string MyIpAddress 
		{
			get { return myIpAddress; }
		}

		public string MySsid 
		{
			get { return mySsid; }
		}

		public Context.EnumServiceType ServiceType 
		{
			get { return serviceType; }
		}

		public int MyAge
		{
			get { return myAge; }
		}

		public Context.EnumGender MyGender 
		{
			get { return myGender; }
		}

		public int MyCarAge
		{
			get { return myCarAge; }
		}

		public string MyCarLicence 
		{
			get { return myCarLicence; }
		}

		public bool CarHasAbs
		{
			get { return carHasAbs; }
		}

		public bool CarHasAirbag
		{
			get { return carHasAirbag; }
		}

		public bool NonSmoker
		{
			get { return nonSmoker; }
		}

		private void parseVps(string vpsXml) 
		{
			MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(vpsXml)); 
			XmlTextReader xmlr = new XmlTextReader(ms); 
			
			// read Pubdata and Privdata in pubXml and privXml
			while (xmlr.Read()) 
			{
				switch (xmlr.NodeType)				
				{
					case XmlNodeType.Element:
						if (xmlr.Name.Equals("Viaticum")) 
						{
							xmlr.MoveToContent();
							xmlr.MoveToAttribute("SigPriv"); 
							while (xmlr.ReadAttributeValue())
							{
								sigPriv = xmlr.Value;
							}
							xmlr.MoveToAttribute("SigPub"); 
							while (xmlr.ReadAttributeValue())
							{
								sigPub = xmlr.Value;
							}
						} 
						else if (xmlr.Name.Equals("Privdata")) 
						{
							privXml = xmlr.ReadOuterXml();
							pubXml = xmlr.ReadOuterXml();
						} 
						break;
					default:
						break;
				}
			}
			// TODO: check signatures of privXml and pubXml

			// read Pubdata
			ms = new MemoryStream(Encoding.UTF8.GetBytes(pubXml)); 
			xmlr = new XmlTextReader(ms); 
			
			while (xmlr.Read()) 
			{
				switch (xmlr.NodeType)				
				{
					case XmlNodeType.Element:
						if (xmlr.Name.Equals("Version")) 
						{
							xmlr.Read();
							version = xmlr.Value;
						}
						else if (xmlr.Name.Equals("ValidFrom")) 
						{
							xmlr.Read();
							validFrom = DateTime.ParseExact(xmlr.Value, "dd.MM.yyyy", CultureInfo.InvariantCulture);
						}
						else if (xmlr.Name.Equals("ValidTo")) 
						{
							xmlr.Read();
							validTo = DateTime.ParseExact(xmlr.Value, "dd.MM.yyyy", CultureInfo.InvariantCulture);
						}
						else if (xmlr.Name.Equals("Id")) 
						{
							xmlr.Read();
							myId = xmlr.Value;
						}
						else if (xmlr.Name.Equals("Image")) 
						{
							xmlr.Read();
							using (MemoryStream hms = new MemoryStream(Convert.FromBase64String(xmlr.Value))) 
							{
								myImage = new Bitmap(hms);
							}
						}
						else if (xmlr.Name.Equals("ServiceType")) 
						{
							xmlr.Read();
							serviceType = Context.string2ServiceType(xmlr.Value);
						}
						else if (xmlr.Name.Equals("Gender")) 
						{
							xmlr.Read();
							myGender = Context.string2gender(xmlr.Value);
						}
						else if (xmlr.Name.Equals("CarLicence")) 
						{
							xmlr.Read();
							myCarLicence = xmlr.Value;
						}
						break;
					default:
						break;
				}
			}

			// read Privdata
			ms = new MemoryStream(Encoding.UTF8.GetBytes(privXml)); 
			xmlr = new XmlTextReader(ms); 
			
			while (xmlr.Read()) 
			{
				switch (xmlr.NodeType)				
				{
					case XmlNodeType.Element:
						if (xmlr.Name.Equals("MasterPk")) 
						{
							masterPubKey = xmlr.ReadInnerXml();
							mySecKey = xmlr.ReadInnerXml();
							xmlr.Read(); // Birthdate
							birthDate = DateTime.ParseExact(xmlr.Value, "dd.MM.yyyy", CultureInfo.InvariantCulture);
							myAge = Context.calcAge(birthDate);
						}
						else if (xmlr.Name.Equals("CarBuilt")) 
						{
							xmlr.Read();
							carBuilt = DateTime.ParseExact(xmlr.Value, "dd.MM.yyyy", CultureInfo.InvariantCulture);
							myCarAge = Context.calcAge(carBuilt);
						}
						else if (xmlr.Name.Equals("HasAirbag")) 
						{
							xmlr.Read();
							carHasAirbag = Context.string2bool(xmlr.Value);
						}
						else if (xmlr.Name.Equals("HasAbs")) 
						{
							xmlr.Read();
							carHasAbs = Context.string2bool(xmlr.Value);
						}
						else if (xmlr.Name.Equals("NonSmoker")) 
						{
							xmlr.Read();
							nonSmoker = Context.string2bool(xmlr.Value);
						}
						else if (xmlr.Name.Equals("SSID")) 
						{
							xmlr.Read();
							mySsid = xmlr.Value;
						}
						else if (xmlr.Name.Equals("IpAddress")) 
						{
							xmlr.Read();
							myIpAddress = xmlr.Value;
						}
						else if (xmlr.Name.Equals("NetMask")) 
						{
							xmlr.Read();
							myNetMask = xmlr.Value;
						}
						else if (xmlr.Name.Equals("MulticastGroup")) 
						{
							xmlr.Read();
							myMulticastGroup = xmlr.Value;
						}
						else if (xmlr.Name.Equals("UdpPort")) 
						{
							xmlr.Read();
							myUdpPort = Convert.ToInt32(xmlr.Value);
						}
						else if (xmlr.Name.Equals("TcpPort")) 
						{
							xmlr.Read();
							myTcpPort = Convert.ToInt32(xmlr.Value);
						}
						break;
					default:
						break;
				}
			}
		}

		public bool verify(string xml, string digest) 
		{
			RsaCrypt rsa = new RsaCrypt(null, this.masterPubKey);			
			return rsa.verifyData(xml, digest);
		}


		public Smartcard(string password)
		{
			string vpsXml;

			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FormAbout));
			Stream file = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("viaticum.sc.vps");
			BinaryReader br = new BinaryReader(file);
			byte[] buf = new byte[file.Length];
			try 
			{
				br.Read(buf, 0, buf.Length);
			} 
			finally 
			{
				br.Close();
			}

			try 
			{
				Des3Crypt des3 = new Des3Crypt(password);
				vpsXml = des3.binary_decrypt(buf);
			}
			catch 
			{
				throw new Exception("das eingegebene Kennwort ist falsch!");	
			}

			utf8 = new System.Text.UTF8Encoding();
			parseVps(vpsXml);
			if (!verify(this.pubXml, this.sigPub)) 
			{
				throw new Exception("Ihre ffentliche Daten sind nicht autorisiert!");
			}
			if (!verify(this.privXml, this.sigPriv)) 
			{
				throw new Exception("Ihre privaten Daten sind nicht autorisiert!");
			}

		}
	}
}
