using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace viaticum
{
	/// <summary>
	/// 
	/// </summary>
	public class TcpDriver
	{
		private TcpListener listener;
		private TcpClient   client;
		private NetworkStream stream;
		private readonly object syncRoot;
		private bool stopped = true;
		private Thread thread;

		public TcpDriver()
		{
			syncRoot = new object();

			listener = new TcpListener(Context.Identcard.MyTcpPort);

			thread = new Thread(new ThreadStart(serverStart));

		}
		public bool IsStopped
		{
			get
			{
				lock(syncRoot)
				{
					return stopped;
				}
			}
		}

		public bool IsStarted
		{
			get
			{
				lock(syncRoot)
				{
					return ! stopped;
				}
			}
		}

		public void Start()
		{
			lock(syncRoot)
			{
				if ( stopped )
				{
					stopped = false;
					thread.Start();
				}
			}
		}

		public void Stop()
		{
			lock(syncRoot)
			{
				if ( stopped ) 
				{
					client = null;
					stream = null;
					return;
				}
				stopped = true;
				shutdown();
				client = null;
				stream = null;
			}
		}

		public bool writeMsg(byte[] dataWrite) 
		{
			if (stream == null || !stream.CanWrite) 
			{
				return false;
			}
			stream.Write(dataWrite, 0, dataWrite.Length);

			return true;
		}

		public bool readMsg(out byte[] dataRead) 
		{
			dataRead = null;

			if (stream == null || !stream.CanRead) 
			{
				return false;
			}

			int size = client.ReceiveBufferSize;

			if (size > 0) 
			{
				dataRead = new byte[size];
				stream.Read(dataRead, 0, size);
			} 

			return (dataRead != null);
		}

		private void serverStart()
		{
			listener.Start();

			try 
			{
				client = listener.AcceptTcpClient(); 
				stream = client.GetStream();
			}
			catch (SocketException se) 
			{ 
				// if listener.Stop() happens
			}
			finally 
			{
				try 
				{
					Stop();
				}
				catch {}
			}

		}

		private void shutdown() 
		{
			if (stream != null) 
			{
				stream.Close();
			}
			if (client != null) 
			{
				client.Close();
			}
			if (listener != null) 
			{
				try 
				{
					listener.Stop();
				}
				catch {}
			}
		}
	}
}
