using System;
using System.Collections;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Text;


namespace viaticum
{
	/// <summary>
	/// Driver part of UDP protocol
	/// </summary>
	public class TcpServer_
	{
		const int c_sleep_time_receive = 5000000;
		const int c_sleep_time = 500;
		bool stopReceiving;
		bool running;
		int serverTcpPort;
		Socket listenerSocket;
		Socket acceptSocket;
		Hashtable ipHash;

		public TcpServer_()
		{
			IPAddress  localIPAddr = IPAddress.Parse(Context.Identcard.MyIpAddress);
			serverTcpPort    = Context.Identcard.MyTcpPort;
			ipHash = new Hashtable();

			try
			{
				// Create a listener socket.
				listenerSocket = new Socket(AddressFamily.InterNetwork, 
					SocketType.Stream, 
					ProtocolType.Tcp);
				              
				// Create an IPEndPoint object. 
				EndPoint localEP = (EndPoint)new IPEndPoint(localIPAddr, serverTcpPort);

				listenerSocket.Blocking = false;
        
				// Bind this endpoint to the Listener socket.
				listenerSocket.Bind(localEP);

				listenerSocket.Listen(10);

			}
			catch (SocketException se)
			{
				string shouldIp = Context.Identcard.MyIpAddress; 
				IPEndPoint myIp = new IPEndPoint(Dns.Resolve(Dns.GetHostName()).AddressList[0], 0);
				Debug.WriteLine("Bitte verwenden Sie die Ihnen zugeordnete IP-Adresse " + shouldIp + " statt der momentan eingestellten " + myIp.Address.ToString());
				//Debug.WriteLine(se.Message);
				shutdown();
				throw new Exception("Konfigurationsfehler", se);
			}		
		}
		public bool StopReceiving
		{ 
			get 
			{
				return stopReceiving;
			}
			set 
			{
				stopReceiving = value;
			}
		}

		private bool Running 
		{ 
			get 
			{
			{return running;}
			}
			set 
			{
				lock(this) 
				{running = value;}
			}
		}

		public bool IsRunning 
		{
			get { return running;}
		}

		public void shutdown () 
		{
			if (listenerSocket != null) 
			{
				StopReceiving = true;
				while (Running)
				{
					Thread.Sleep(c_sleep_time);
				}
				listenerSocket.Shutdown(SocketShutdown.Both);
				listenerSocket.Close();
				listenerSocket = null;
			}
		}
		private bool check_xml_msg (byte[] bytes) 
		{ 
			int ageFrom;
			int ageTo;
			int reqMaxNumPers;
			bool canSmoke;
			Context.EnumGender gender;
			Context.EnumDirections direction;
			XMLPackaging xp = new XMLPackaging();

			xp.xml2ave(bytes, out ageFrom, out ageTo, out reqMaxNumPers, out canSmoke, out gender, out direction);

			return (Context.Identcard.MyAge >= ageFrom && Context.Identcard.MyAge <= ageTo
				&&
				(!Context.ReqMaySmoke || canSmoke)
				&&
				(gender == Context.EnumGender.unspezifiziert ||
				Context.Identcard.MyGender == gender)
				&&
				direction == Context.ReqDirection
				); 
		}

		public void startReceiveRequests() 
		{
			if (Running) 
			{
				return;
			}
			Thread tReceive = new Thread(new ThreadStart(receiveRequest));
			StopReceiving = false;
			tReceive.Start();
		}

		private void receiveRequest() 
		{
			//IPEndPoint ipAny = new IPEndPoint(IPAddress.Any, serverTcpPort);
			ArrayList readList = new ArrayList();
			int available = 0;

			Running = true;
			while (!StopReceiving) 
			{
				//listenerSocket.Blocking = false;
				readList.Add(listenerSocket);
				Socket.Select(readList, null, null, c_sleep_time_receive);
				if (readList.Count != 0) // socket activity
				{
					acceptSocket = ((Socket)readList[0]).Accept();
					readList.Clear();
					readList.Add(acceptSocket);
					Socket.Select(readList, null, null, c_sleep_time_receive);

					if (readList.Count != 0) // socket activity
					{
						available = acceptSocket.Available;
						if (available > 0) 
						{
							byte[] bytes = new Byte[available];
							//remoteEP = (EndPoint)ipAny;
							//Debug.WriteLine("vor ReceiveFrom, available = " + available);
							//listenerSocket.Blocking = true;
						//listenerSocket.ReceiveFrom(bytes, available, SocketFlags.None, ref remoteEP);
							//Debug.WriteLine("nach ReceiveFrom =");
							available = acceptSocket.Receive(bytes, 0, available, SocketFlags.None);
							string remoteIp = acceptSocket.RemoteEndPoint.ToString();
							Debug.WriteLine("Received broadcast from " + remoteIp);
							if (bytes.Length > 0) 
							{
								//string xml_msg = Encoding.UTF8.GetString(bytes,0,bytes.Length).ToString();
								if (!ipHash.ContainsKey(remoteIp)) 
								{
									ipHash.Add(remoteIp, 1);
									if (check_xml_msg(bytes)) 
									{
										Context.addPartnerAddress(remoteIp);
									}
								}
							}
						}
						readList.Clear();
					}
				}
				Running = false;
			}
		}
	}
}