using System;
using System.Collections;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace viaticum
{
	/// <summary>
	/// Driver part of UDP protocol
	/// </summary>
	public class UdpSender
	{
		const int c_sleep_time_shutdown = 500;
		const int c_sleep_time_sending = 2000;
		IPAddress mcastAddress;
		int mcastPort;
		Socket mcastSocket;
		MulticastOption mcastOption;
		IPEndPoint endPoint;
		bool stopSending;
		bool running;
	
		public UdpSender()
		{	//return; // Test

			// Initialize the multicast address group and multicast port.
			// Both address and port are selected from the allowed sets as
			// defined in the related RFC documents. These are the same 
			// as the values used by the sender.
			
			// Get the local IP address used by the listener and the sender to
			// exchange multicast messages. 
			IPAddress  localIPAddr = IPAddress.Parse(Context.Identcard.MyIpAddress);
			// Create an IPEndPoint object. 
			IPEndPoint IPlocal = new IPEndPoint(localIPAddr, 0);
			mcastAddress = IPAddress.Parse(Context.Identcard.MyMulticastGroup);
			mcastPort    = Context.Identcard.MyUdpPort;

			// join multicast group
			try
			{
				// Create a multicast socket.
				mcastSocket = new Socket(AddressFamily.InterNetwork, 
					SocketType.Dgram, 
					ProtocolType.Udp);
              
				// Bind this endpoint to the multicast socket.
				mcastSocket.Bind(IPlocal);

				// Define a MulticastOption object specifying the multicast group 
				// address and the local IP address.
				// The multicast group address is the same as the address used by the listener.
				mcastOption = new MulticastOption(mcastAddress, localIPAddr);
        
				mcastSocket.SetSocketOption(SocketOptionLevel.IP, 
					SocketOptionName.AddMembership, 
					mcastOption);

				endPoint = new IPEndPoint(mcastAddress, mcastPort);
     
			}
			catch (SocketException se)
			{
				string shouldIp = IPlocal.Address.ToString(); 
				IPEndPoint myIp = new IPEndPoint(Dns.Resolve(Dns.GetHostName()).AddressList[0], 0);
				Debug.WriteLine("Bitte verwenden Sie die Ihnen zugeordnete IP-Adresse " + shouldIp + " statt der momentan eingestellten " + myIp.Address.ToString());
				shutdown();
				throw new Exception("Konfigurationsfehler", se);
			}		
		}
		public void shutdown () 
		{
			if (mcastSocket != null) 
			{
				StopSending = true;
				while (Running)
				{
					Thread.Sleep(c_sleep_time_shutdown);
				}
				mcastSocket.Shutdown(SocketShutdown.Both);
				mcastSocket.Close();
				mcastSocket = null;
			}
		}

		public bool StopSending 
		{ 
			get 
			{
				return stopSending;
			}
			set 
			{
				stopSending = value;
			}
		}

		private bool Running 
		{ 
			get 
			{
				{return running;}
			}
			set 
			{
				lock(this) 
				{running = value;}
			}
		}

		public bool IsRunning 
		{
			get { return running;}
		}

		public void broadcastAVE()
		{
			if (Running) 
			{
				return;
			}
			Thread tSend = new Thread(new ThreadStart(sendAVE));
			StopSending = false;
			tSend.Start();
		}

		public void sendAVE()
		{
			Running = true;
			XMLPackaging pkg = new XMLPackaging();
			byte[] message = pkg.ave2xml(Context.ReqAgeFrom, Context.ReqAgeTo, Context.ReqMaxNumPers, !Context.ReqNonSmoker, Context.Identcard.MyGender, Context.ReqDirection);
			while (!StopSending) 
			{
				try
				{
					//Send multicast packets to the listener.
					string msg = Encoding.UTF8.GetString(message,0,message.Length).ToString();
					//Debug.WriteLine("sende: " + msg);
					mcastSocket.SendTo(message, message.Length, SocketFlags.None, endPoint);            
				}
				catch (Exception e)
				{
					Running = false;
					throw(e);
				}
				Thread.Sleep(c_sleep_time_sending);
			}
			Running = false;
		}
	}
}
