using System;
using System.IO; 
using System.Text; 
using System.Xml; 

namespace viaticum
{
	/*
	 * implementiert Paketierung fr folgendes Protokoll
	 * 
	 * AVE, VIA, OFR, ITA
	 * 
	 * Client                                                                      Server
	 * ------                                                                      ------
	 * 
	 *                       ReqAgeFrom, ReqAgeTo, ReqCanSmoke, ReqGender,
	 *                       ReqDirection, ReqMaxNumPers
	 * Driver    -----------------------------------------------------------------> Passenger
	 *                                           AVE/UDP
	 * 
	 * 
	 *                       PubXml, SigPub, ReqDestination1, ReqDestination2, 
	 *                       ReqDestination2Optional, ReqGender, ReqAgeFrom, 
	 *                       ReqAgeTo, ReqNonSmoker, ReqMaySmoke, ReqAgeCar,
	 *                       ReqHasAbs, ReqHasAirbag, ReqSeat                       
	 * Passenger -----------------------------------------------------------------> Driver
	 *                                           VIA/TCP
	 * 
	 * 
	 *                       PubXml, SigPub                       
	 * Passenger <----------------------------------------------------------------- Driver
	 *                                           OFR/TCP
	 * 
	 * 
	 *                       Y/N                       
	 * Passenger <----------------------------------------------------------------- Driver
	 *                                           ITA/TCP
	 */
	public class XMLPackaging
	{
		UTF8Encoding utf8;

		public XMLPackaging()
		{
			utf8 = new System.Text.UTF8Encoding();
		}

		public byte[] ave2xml(int ageFrom, int ageTo, int reqMaxNumPers, bool canSmoke, Context.EnumGender gender, 
			Context.EnumDirections direction) 
		{
			MemoryStream ms = new MemoryStream(); 
			XmlTextWriter xmlw = new XmlTextWriter(ms, utf8); 

			xmlw.WriteStartDocument();

			// root element
			xmlw.WriteStartElement("AVE");

			//message content
			xmlw.WriteElementString("af", ageFrom.ToString());
			xmlw.WriteElementString("at", ageTo.ToString());
			xmlw.WriteElementString("p", reqMaxNumPers.ToString());
			xmlw.WriteElementString("d", Context.direction2string(direction));
			xmlw.WriteElementString("g", Context.gender2string(gender));
			xmlw.WriteElementString("s", Context.bool2string(canSmoke));

			// close message
			xmlw.WriteEndElement();
			xmlw.WriteEndDocument();
			xmlw.Flush();
			xmlw.Close();

			return (ms.ToArray());

		}

		public void xml2ave(byte[] bytes, out int ageFrom, out int ageTo, out int reqMaxNumPers, out bool canSmoke, out Context.EnumGender gender, out Context.EnumDirections direction) 
		{
			MemoryStream ms = new MemoryStream(bytes); 
			XmlTextReader xmlr = new XmlTextReader(ms); 
			ageFrom = Context.minAge;
			ageTo = Context.maxAge;
			reqMaxNumPers = Context.defMaxNumPers;
			canSmoke = true;
			gender = Context.EnumGender.unspezifiziert;
			direction = Context.EnumDirections.Westen;

			while (xmlr.Read()) 
			{
				switch (xmlr.NodeType)				
				{
					case XmlNodeType.Element:
						if (xmlr.Name.Equals("af")) 
						{
							xmlr.Read();
							ageFrom = Convert.ToInt32(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("at")) 
						{
							xmlr.Read();
							ageTo = Convert.ToInt32(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("p")) 
						{
							xmlr.Read();
							reqMaxNumPers = Convert.ToInt32(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("d")) 
						{
							xmlr.Read();
							direction = Context.string2direction(xmlr.Value);
						}
						else if (xmlr.Name.Equals("g")) 
						{
							xmlr.Read();
							gender = Context.string2gender(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("s")) 
						{
							xmlr.Read();
							canSmoke = Context.string2bool(xmlr.Value);
						} 

						break;
					default:
						break;
				}
			}
		}

		public byte[] via2xml(string pubXml, string sigPub, string reqDestination1, string reqDestination2,
			bool reqDestination2Optional, Context.EnumGender reqGender, int reqAgeFrom, int reqAgeTo, 
			bool reqNonSmoker, bool reqMaySmoke, int reqAgeCar, bool reqHasAbs, bool reqHasAirbag, 
			Context.EnumSeat reqSeat) 
		{
			MemoryStream ms = new MemoryStream(); 
			XmlTextWriter xmlw = new XmlTextWriter(ms, utf8); 

			xmlw.WriteStartDocument();

			// root element
			xmlw.WriteStartElement("VIA");

			//message content
			xmlw.WriteElementString("sp", sigPub);
			xmlw.WriteElementString("d1", reqDestination1);
			xmlw.WriteElementString("d2", reqDestination2);
			xmlw.WriteElementString("do", Context.bool2string(reqDestination2Optional));
			xmlw.WriteElementString("g",  Context.gender2string(reqGender));
			xmlw.WriteElementString("af", reqAgeFrom.ToString());
			xmlw.WriteElementString("at", reqAgeTo.ToString());
			xmlw.WriteElementString("ns", Context.bool2string(reqNonSmoker));
			xmlw.WriteElementString("ms", Context.bool2string(reqMaySmoke));
			xmlw.WriteElementString("ac", reqAgeCar.ToString());
			xmlw.WriteElementString("ha", Context.bool2string(reqHasAbs));
			xmlw.WriteElementString("ai", Context.bool2string(reqHasAirbag));
			xmlw.WriteElementString("s",  Context.seat2string(reqSeat));
			xmlw.WriteRaw(pubXml);

			// close message
			xmlw.WriteEndElement();
			xmlw.WriteEndDocument();
			xmlw.Flush();
			xmlw.Close();

			return (ms.ToArray());
		}

		public void xml2via(byte[] bytes, out string pubXml, out string sigPub, out string reqDestination1, 
			out string reqDestination2, out bool reqDestination2Optional, out Context.EnumGender reqGender, 
			out int reqAgeFrom, out int reqAgeTo, out bool reqNonSmoker, out bool reqMaySmoke, out int reqAgeCar, 
			out bool reqHasAbs, out bool reqHasAirbag, out Context.EnumSeat reqSeat)
		{
			MemoryStream ms = new MemoryStream(bytes); 
			XmlTextReader xmlr = new XmlTextReader(ms); 
			pubXml = sigPub = reqDestination1 = reqDestination2 = null;
			reqGender = Context.EnumGender.unspezifiziert;
			reqAgeFrom = Context.minAge;
			reqAgeTo = Context.maxAge;
			reqDestination2Optional = reqNonSmoker = reqHasAirbag = reqHasAbs = false;
			reqMaySmoke = true;
			reqAgeCar = Context.maxCarAge;
			reqSeat = Context.EnumSeat.unspezifiziert;

			while (xmlr.Read()) 
			{
				switch (xmlr.NodeType)				
				{
					case XmlNodeType.Element:
						if (xmlr.Name.Equals("sp")) 
						{
							xmlr.Read();
							sigPub = xmlr.Value;
						} 
						else if (xmlr.Name.Equals("d1")) 
						{
							xmlr.Read();
							reqDestination1 = xmlr.Value;
						} 
						else if (xmlr.Name.Equals("d2")) 
						{
							xmlr.Read();
							reqDestination2 = xmlr.Value;
						} 
						else if (xmlr.Name.Equals("do")) 
						{
							xmlr.Read();
							reqDestination2Optional = Context.string2bool(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("g")) 
						{
							xmlr.Read();
							reqGender = Context.string2gender(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("af")) 
						{
							xmlr.Read();
							reqAgeFrom = Convert.ToInt32(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("at")) 
						{
							xmlr.Read();
							reqAgeTo = Convert.ToInt32(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("ns")) 
						{
							xmlr.Read();
							reqNonSmoker = Context.string2bool(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("ms")) 
						{
							xmlr.Read();
							reqMaySmoke = Context.string2bool(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("ac")) 
						{
							xmlr.Read();
							reqAgeCar = Convert.ToInt32(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("ha")) 
						{
							xmlr.Read();
							reqHasAbs = Context.string2bool(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("ai")) 
						{
							xmlr.Read();
							reqHasAirbag = Context.string2bool(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("s")) 
						{
							xmlr.Read();
							reqSeat = Context.string2seat(xmlr.Value);
						} 
						else if (xmlr.Name.Equals("Pubdata")) 
						{
							pubXml = xmlr.ReadOuterXml();
						} 

						break;
					default:
						break;
				}
			}
		}


		public byte[] ofr2xml(string pubXml, string sigPub) 
		{
			MemoryStream ms = new MemoryStream(); 
			XmlTextWriter xmlw = new XmlTextWriter(ms, utf8); 

			xmlw.WriteStartDocument();

			// root element
			xmlw.WriteStartElement("OFR");

			//message content
			xmlw.WriteElementString("sp", sigPub);
			xmlw.WriteRaw(pubXml);

			// close message
			xmlw.WriteEndElement();
			xmlw.WriteEndDocument();
			xmlw.Flush();
			xmlw.Close();

			return (ms.ToArray());
		}

		public void xml2ofr(byte[] bytes, out string pubXml, out string sigPub)
		{
			MemoryStream ms = new MemoryStream(bytes); 
			XmlTextReader xmlr = new XmlTextReader(ms); 
			pubXml = sigPub = null;

			while (xmlr.Read()) 
			{
				switch (xmlr.NodeType)				
				{
					case XmlNodeType.Element:
						if (xmlr.Name.Equals("sp")) 
						{
							xmlr.Read();
							sigPub = xmlr.Value;
						} 
						else if (xmlr.Name.Equals("Pubdata")) 
						{
							pubXml = xmlr.ReadOuterXml();
						} 

						break;
					default:
						break;
				}
			}
		}

		public byte[] ita2xml(bool yesNo) 
		{
			MemoryStream ms = new MemoryStream(); 
			XmlTextWriter xmlw = new XmlTextWriter(ms, utf8); 

			xmlw.WriteStartDocument();

			// root element
			xmlw.WriteStartElement("ITA");

			//message content
			xmlw.WriteElementString("a", Context.bool2string(yesNo));

			// close message
			xmlw.WriteEndElement();
			xmlw.WriteEndDocument();
			xmlw.Flush();
			xmlw.Close();

			return (ms.ToArray());
		}
		public void xml2ita(byte[] bytes, out bool yesNo)
		{
			MemoryStream ms = new MemoryStream(bytes); 
			XmlTextReader xmlr = new XmlTextReader(ms); 
			yesNo = false;

			while (xmlr.Read()) 
			{
				switch (xmlr.NodeType)				
				{
					case XmlNodeType.Element:
						if (xmlr.Name.Equals("a")) 
						{
							xmlr.Read();
							yesNo = Context.string2bool(xmlr.Value);
						} 

						break;
					default:
						break;
				}
			}
		}

	}
}
